package	jp.co.sds_corp.hibiya.monitor;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.Vector;

import javax.swing.JComponent;

import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.nanchart.graph.Graph2D;
import jp.co.sds_corp.nanchart.graph.GraphSeries;
import jp.co.sds_corp.nanchart.graph.GraphStorage;
import jp.co.sds_corp.nanchart.graph.LineGraph;
import jp.co.sds_corp.nanchart.graph.LineStackGraph;
import jp.co.sds_corp.nanchart.graph.StairGraph;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.util.common.CmnDate;

public class Graph extends JComponent implements ICheckerListener,
										GraphLegend.SelectionListener
{
	private static final long serialVersionUID = 1L;
	GraphSetupParam	setup;
	java.util.List<GraphParam>	listParam;
	Vector<GraphBuffer>			vecBuf = new Vector<GraphBuffer>();
	GraphLegend		legend;
	DecimalFormat df1 = new DecimalFormat( "00" );
	int		nMax;
	int		selected = -1;
	GraphSeries grf;

	public Graph() {
		legend = new GraphLegend();
		legend.addSelectionListener( this );
	}

	public void setParam( GraphSetupParam setup, java.util.List<GraphParam> list ) {
		this.setup = setup;
		for ( int i = 0; i < vecBuf.size(); i++ ) {
			GraphBuffer buf = (GraphBuffer )vecBuf.get( i );
			buf.removeCheckerListener( this );
		}
		vecBuf.clear();
		this.listParam = list;
		for ( int i = 0; i < list.size(); i++ ) {
			GraphParam prm = (GraphParam )list.get( i );
			GraphBuffer buf = GraphBufferFactory.getBuffer(
				prm.getConnector(), prm.getChecker(), prm );
			vecBuf.add( buf );
			buf.addCheckerListener( this );
		}
		legend.setParam( listParam );
		if ( list.size() == 0 ) {
			selected = -1;
		} else { 
			selected = 0;
		}
	}

	public GraphLegend getLegend() {
		return legend;
	}
	public void selectionChanged( int var ) {
		setSelected( var );
		repaint();
	}

	public void setSelected( int var ) {
		this.selected = var;
	}
	public int getSelected() {
		if ( listParam.size() < 1 ) {
			return -1;
		}
		if ( listParam.size() == 1 ) {
			return 0;
		}
		if ( setup.getType() == GraphSetupParam.Type.STACK ) {
			return 0;
		}
		return selected;
	}

	/**
	* ICheckerの状態に変更があった場合に呼び出されます。
	* @param	ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		repaint();
	}

	public void paint( Graphics gr ) {
		CmnDate fr, to;

		if ( vecBuf.size() == 0 ) {
			return;
		}
		CmnDate last = ((GraphBuffer )vecBuf.get( 0 )).getLastUpdated();
		fr = (CmnDate )last.clone();
		to = (CmnDate )last.clone();

		switch ( setup.getTerm()) {
		case MIN15:
			fr.add( CmnDate.MINUTE, -15 );
			break;
		case HOUR:
			fr.add( CmnDate.HOUR, -1 );
			break;
		case DATE:
			fr.add( CmnDate.DATE, -1 );
			break;
		}

		if ( getSelected() >= 0 ) {
			for ( int i = 0; i < listParam.size(); i++ ) {
				GraphParam gprm = (GraphParam )listParam.get( i );
				GraphBuffer buf = GraphBufferFactory.getBuffer(
					gprm.getConnector(), gprm.getChecker(), gprm );
				if ( i == getSelected() ) {
					calcMinMax( gprm, buf );
					break;
				}
			}
		}
		Graph2D g2d = new Graph2D( fr, to, new Double( fMin ), new Double( fMax ));
		grf = null;
		for ( int i = 0; i < listParam.size(); i++ ) {
			GraphParam gprm = (GraphParam )listParam.get( i );
			GraphBuffer buf = GraphBufferFactory.getBuffer(
				gprm.getConnector(), gprm.getChecker(), gprm );
			drawSeries( g2d, gprm, buf, fr, to );
			if ( i == getSelected() ) {
				YScale  ys = (YScale )g2d.getYScale();
				setYScale( ys, (long )fMin, (long )fMax, gprm );
			}
		}
		XScale  xs = (XScale )g2d.getXScale();
		setXScale( xs, fr, to );

		Rectangle rect = new Rectangle( 25, 5,
							(int )getWidth() - 25, (int )getHeight() - 15 );
		g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
							RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.drawGraph( (Graphics2D )gr, rect );
	}

	double fMax;
	double fMin;
	protected void calcMinMax( GraphParam gprm, GraphBuffer buf ) {
		DecimalFormat fmt;

		fMax = Double.MIN_VALUE;
		fMin = Double.MAX_VALUE;
		fmt = gprm.getFormat();
		if ( gprm.getMax() == null || gprm.getMin() == null ) {
			for ( int i = 0; i < buf.size(); i++ ) {
				GraphBuffer.Data data = (GraphBuffer.Data )buf.get( i );
				if ( data.getData() == null ) {
					return;
				}
				try {
					double fVal = fmt.parse( (String )data.getData() ).doubleValue();
					if ( fVal > fMax ) {
						fMax = fVal;
					}
					if ( fVal < fMin ) {
						fMin = fVal;
					}
				} catch ( ParseException ex ) {
				}
			}
		} else {
			fMax = gprm.getMax().doubleValue();
			fMin = gprm.getMin().doubleValue();
		}
	}

	protected void drawSeries( Graph2D g2d, GraphParam gprm, GraphBuffer buf,
								CmnDate fr, CmnDate to )
	{
		DecimalFormat fmt;

		fmt = gprm.getFormat();
		calcMinMax( gprm, buf );

		switch ( setup.getType()) {
		case STAIR:
			grf = new StairGraph( fr, to, new Double( fMin ), new Double( fMax ));
			g2d.addSeries( grf );
			break;
		case LINE:
			grf = new LineGraph( fr, to, new Double( fMin ), new Double( fMax ));
			g2d.addSeries( grf );
			break;
		case STACK:
			if ( grf == null ) {
				// grf = new AreaStackGraph( fr, to, new Double( fMin ), new Double( fMax ));
				grf = new LineStackGraph( fr, to, new Double( fMin ), new Double( fMax ));
				g2d.addSeries( grf );
			}
			break;
		}
		grf.setStroke(
			new BasicStroke( 1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL ) );
		Color col = gprm.getColor();
		switch ( setup.getType()) {
		case STACK:
			grf.setLineColor( gprm.getValue(), col );
			grf.setFillColor( gprm.getValue(), col );
			break;
		default:
			grf.setLineColor( col );
			grf.setFillColor( col );
		}
		GraphStorage store = grf.getStorage();
		for ( int i = 0; i < buf.size(); i++ ) {
			GraphBuffer.Data data = (GraphBuffer.Data )buf.get( i );
			if ( data.getData() == null ) {
				return;
			}
			if ( data.getTime().before( fr )) {
				continue;
			}
			try {
				Number val = fmt.parse( (String )data.getData() );
				switch ( setup.getType()) {
				case STACK:
					store.add( gprm.getValue(), data.getTime(), val );
					break;
				default:
					store.add( data.getTime(), val );
				}
			} catch ( ParseException ex ) {
			}
		}
	}

	protected void setXScale( XScale xs, CmnDate fr, CmnDate to ) {
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		xs.setFont( font );
		CmnDate tmp = (CmnDate )fr.clone();
		tmp.set( CmnDate.SECOND, 0 );
		tmp.set( CmnDate.MINUTE, 0 );
		for ( ;; ) {
			switch ( setup.getTerm() ) {
			case DATE:
				tmp.add( CmnDate.HOUR, 1 );
				break;
			case HOUR:
			case MIN15:
				tmp.add( CmnDate.MINUTE, 1 );
				break;
			}
			if ( tmp.after( to ) ) {
				break;
			}
			switch ( setup.getTerm() ) {
			case DATE:
				if ( tmp.get( CmnDate.HOUR ) % 6 == 0 ) {
					xs.add( tmp.clone(), tmp.format( "HH" ));
				} else {
					xs.add( tmp.clone(), "" );
				}
				break;
			case HOUR:
				if ( tmp.get( CmnDate.MINUTE ) % 10 == 0 ) {
					xs.add( tmp.clone(), tmp.format( "mm" ));
				} else if ( tmp.get( CmnDate.MINUTE ) % 5 == 0 ) {
					xs.add( tmp.clone(), "" );
				}
				break;
			case MIN15:
				if ( tmp.get( CmnDate.MINUTE ) % 5 == 0 ) {
					xs.add( tmp.clone(), tmp.format( "mm" ));
				} else {
					xs.add( tmp.clone(), "" );
				}
				break;
			}
		}
		xs.setScaleLine( true );
		xs.setLineColor( Color.gray );
	}

	protected void setYScale( YScale ys, long nMin, long nMax, GraphParam gprm )
	{
		if ( nMin == (long )Double.MAX_VALUE ) {
			return;
		}
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		ys.setFont( font );
		String sc;
		sc = gprm.getFormat().format( nMin );
		ys.add( nMin, sc );
		sc = gprm.getFormat().format( nMax );
		ys.add( nMax, sc );

		String s = String.valueOf( nMax );
		long nAdd = (int )Math.pow( 10, s.length() - 2 );
		long nDiv = (int )Math.pow( 10, s.length() - 1 );
		if ( nDiv > 1 && (( nMax - nMin ) / nDiv ) < 5 ) {
			nDiv /= 2;
		} else {
			nAdd *= 5;
		}
/*
System.out.println( "min=" + nMin + " max=" + nMax
+ " add=" + nAdd + " div=" + nDiv );
*/
		if ( nAdd <= 0 ) {
			return;
		}
		long nStart = (( nMin - 1 ) / nAdd + 1 ) * nAdd;
		for ( long i = nStart; i <= nMax; i += nAdd ) {
			if (( i % nDiv ) == 0 || i == nMin || i == nMax ) {
				sc = gprm.getFormat().format( i );
				ys.add( i, sc );
			} else {
				ys.add( i, "" );
			}
		}
		ys.setScaleLine( true );
		ys.setLineColor( Color.gray );
	}

}
