package jp.co.sds_corp.hibiya.monitor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* グラフの設定を行います。
*/
public class GraphSetupDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	SetupParam		setup;
	GraphSetupParam	gsu;
	java.util.List<GraphParam>	listParam;
	ResourceBundle	res;
	IConnector		con;
	IChecker		chk;

	JRadioButton	RBT_TYPE1, RBT_TYPE2, RBT_TYPE3, RBT_TYPE4;
	JRadioButton	RBT_TERM1, RBT_TERM2, RBT_TERM3, RBT_TERM4;
	JScrollPane	sp;
	ParamModel	model;
	JTable		table;
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	JPopupMenu	rMenu;
	int		nRes = -1;

	public GraphSetupDialog( Frame parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;
		createControls();
	}

	public GraphSetupDialog( Dialog parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;
		createControls();
	}

	private void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		String text;
		setModal( true );
		setResizable( false );
		text = res.getString( "graph.setup.title" );
		setTitle( text );
		setSize( 477, 421 );
		getContentPane().setLayout(
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));
		Font	f = new Font( "Dialog", Font.PLAIN, 14 );
		setFont( f );
		WindowUtility.setHideOnEscape( this );

		JPanel panel;
		panel = createTypePanel();
		getContentPane().add( panel );
		panel = createTermPanel();
		getContentPane().add( panel );
		createSheet();
		getContentPane().add( sp );

		Box box1 = new Box( BoxLayout.X_AXIS );
		box1.add( BTN_OK );
		box1.add( BTN_CANCEL );
		getContentPane().add( box1 );
		getRootPane().setDefaultButton( BTN_OK );

		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onOk();
			}
		});
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onCancel();
			}
		});
		createPopup();
	}

	private JPanel createTypePanel() {
		ButtonGroup grp = new ButtonGroup();
		String text;
		JPanel panel = new JPanel();
		text = res.getString( "graph.setup.group1.title" );
		panel.setBorder( new TitledBorder( text ));

		text = res.getString( "graph.setup.type1.text" );
		RBT_TYPE1 = new JRadioButton( text );
		panel.add( RBT_TYPE1 );
		grp.add( RBT_TYPE1 );
		text = res.getString( "graph.setup.type2.text" );
		RBT_TYPE2 = new JRadioButton( text );
		panel.add( RBT_TYPE2 );
		grp.add( RBT_TYPE2 );
		text = res.getString( "graph.setup.type3.text" );
		RBT_TYPE3 = new JRadioButton( text );
		panel.add( RBT_TYPE3 );
		grp.add( RBT_TYPE3 );
		return panel;
	}

	private JPanel createTermPanel() {
		ButtonGroup grp = new ButtonGroup();
		String text;
		JPanel panel = new JPanel();
		text = res.getString( "graph.setup.group2.title" );
		panel.setBorder( new TitledBorder( text ));

		text = res.getString( "graph.setup.term1.text" );
		RBT_TERM1 = new JRadioButton( text );
		grp.add( RBT_TERM1 );
		panel.add( RBT_TERM1 );
		text = res.getString( "graph.setup.term2.text" );
		RBT_TERM2 = new JRadioButton( text );
		grp.add( RBT_TERM2 );
		panel.add( RBT_TERM2 );
		text = res.getString( "graph.setup.term3.text" );
		RBT_TERM3 = new JRadioButton( text );
		grp.add( RBT_TERM3 );
		panel.add( RBT_TERM3 );
		return panel;
	}

	private void createSheet() {
		model = new ParamModel();
		table = new JTable( model );
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		table.setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		table.setSurrendersFocusOnKeystroke( true );
		table.addMouseListener( new MouseAdapter() {
/*
			public void mouseReleased( MouseEvent ev ) {
				sheetClicked( ev );
			}
*/
			public void mouseClicked( MouseEvent ev ) {
				sheetClicked( ev );
			}
		});
		TableColumn col;
		col = table.getColumnModel().getColumn( 0 );
		col.setMaxWidth( 20 );
		col.setCellRenderer( new ParamRenderer() );
		col = table.getColumnModel().getColumn( 1 );
		sp = new JScrollPane( table );
		sp.addMouseListener( new MouseAdapter() {
			public void mouseReleased( MouseEvent ev ) {
				if ( SwingUtilities.isRightMouseButton( ev )) {
					showPopup( sp, -1, ev );
				}
			}
		});
	}

	protected void sheetClicked( MouseEvent ev ) {
		int row = table.rowAtPoint( new Point( ev.getX(), ev.getY() ));
		int col = table.columnAtPoint( new Point( ev.getX(), ev.getY() ));
		if ( SwingUtilities.isRightMouseButton( ev )) {
			showPopup( table, row, ev );
		}
		if ( col != 0 ) {
			return;
		}
		Color color = (Color )((Vector )model.getDataVector().get( row )).get( col );
		// Window win = SwingUtilities.windowForComponent( this );
		Color sel = JColorChooser.showDialog( this, "select color", color );
		if ( sel == null ) {
			return;
		}
		((Vector<Color> )model.getDataVector().get( row )).set( col, sel );
	}

	private void createPopup() {
		rMenu = new JPopupMenu();
		JMenuItem   item;
		String  text;
		text = res.getString( "graph.setup.menu.item1.text" );
		item = new JMenuItem( text, KeyEvent.VK_A );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				addParam();
			}
		});
		rMenu.add( item );
		text = res.getString( "graph.setup.menu.item2.text" );
		item = new JMenuItem( text, KeyEvent.VK_D );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				delParam();
			}
		});
		rMenu.add( item );
	}

	protected void addParam() {
		Object[] row = { Color.black, "", "", null, null };
		model.addRow( row );
	}
	protected void delParam() {
		int idx = table.getSelectedRow();
		if ( idx < 0 ) {
			return;
		}
		model.removeRow( idx );
	}

	protected void showPopup( Component c, int row, MouseEvent ev ) {
		if ( row < 0 || row >= table.getRowCount()) {
			row = -1;
		} else {
			table.setRowSelectionInterval( row, row );
		}

		JMenuItem itemDel = (JMenuItem )rMenu.getComponent( 1 );
		if ( row < 0 ) {
			itemDel.setEnabled( false );
		} else {
			itemDel.setEnabled( true );
		}
		rMenu.show( c, ev.getX(), ev.getY() );
	}

	public void setParam( IConnector con, IChecker chk,
						GraphSetupParam gsu, java.util.List<GraphParam> list )
	{
		this.con = con;
		this.chk = chk;
		this.gsu = gsu;
		listParam = list;
		model.fireTableDataChanged();
	}

	public void setVisible( boolean var ) {
		if ( var ) {
			model.getDataVector().clear();
			for ( int i = 0; i < listParam.size(); i++ ) {
				GraphParam prm = (GraphParam )listParam.get( i );
				Object[] row = new Object[model.getColumnCount()];
				row[0] = prm.getColor();
				row[1] = prm.getValue();
				row[2] = prm.getFormat().toPattern();
				if ( prm.getMin() != null ) {
					row[3] = prm.getFormat().format( prm.getMin());
				}
				if ( prm.getMax() != null ) {
					row[4] = prm.getFormat().format( prm.getMax());
				}
				model.addRow( row );
			}
			model.fireTableDataChanged();

			switch ( gsu.getType()) {
			case STAIR:
				RBT_TYPE1.setSelected( true );
				break;
			case LINE:
				RBT_TYPE2.setSelected( true );
				break;
			case STACK:
				RBT_TYPE3.setSelected( true );
				break;
			}
			switch ( gsu.getTerm()) {
			case MIN15:
				RBT_TERM1.setSelected( true );
				break;
			case HOUR:
				RBT_TERM2.setSelected( true );
				break;
			case DATE:
				RBT_TERM3.setSelected( true );
				break;
			}
		}
		super.setVisible( var );
	}

	protected boolean checkInput() {
		boolean bRet = true;
		DecimalFormat fmt = new DecimalFormat();
		String errs = "";
		Vector rows = model.getDataVector();
		for ( int i = 0; i < rows.size(); i++ ) {
			Vector<String> cols = (Vector<String> )rows.get( i );
			fmt.applyPattern( (String )cols.get( 2 ));
			String val;
			val = (String )cols.get( 3 );
			if ( val != null && val.length() > 0 ) {
				try {
					fmt.parse( val );
				} catch ( ParseException ex ) {
					bRet = false;
					errs += model.getColumnName( 3 ) + " [" + val + "]\n";
				}
			}
			val = (String )cols.get( 4 );
			if ( val != null && val.length() > 0 ) {
				try {
					fmt.parse( val );
				} catch ( ParseException ex ) {
					bRet = false;
					errs += model.getColumnName( 4 ) + " [" + val + "]\n";
				}
			}
		}
		if ( !bRet ) {
			String title = res.getString( "graph.setup.err1.title" );
			String msg = res.getString( "graph.setup.err1.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + errs, title, JOptionPane.ERROR_MESSAGE );
		}
		return bRet;
	}

	public void onOk() {
		if ( table.getCellEditor() != null ) {
			table.getCellEditor().stopCellEditing();
		}
		if ( !checkInput()) {
			return;
		}
		if ( RBT_TYPE1.isSelected()) {
			gsu.setType( GraphSetupParam.Type.STAIR );
		} else if ( RBT_TYPE2.isSelected()) {
			gsu.setType( GraphSetupParam.Type.LINE );
		} else if ( RBT_TYPE3.isSelected()) {
			gsu.setType( GraphSetupParam.Type.STACK );
		}
		if ( RBT_TERM1.isSelected()) {
			gsu.setTerm( GraphSetupParam.Term.MIN15 );
		} else if ( RBT_TERM2.isSelected()) {
			gsu.setTerm( GraphSetupParam.Term.HOUR );
		} else if ( RBT_TERM3.isSelected()) {
			gsu.setTerm( GraphSetupParam.Term.DATE );
		}
		Vector rows = model.getDataVector();
		listParam.clear();
		for ( int i = 0; i < rows.size(); i++ ) {
			Vector cols = (Vector )rows.get( i );
			GraphParam prm = new GraphParam();
			prm.setColor( (Color )cols.get( 0 ));
			prm.setValue( (String )cols.get( 1 ));
			prm.setFormat( new DecimalFormat( (String )cols.get( 2 )));
			String val;
			val = (String )cols.get( 3 );
			if ( val != null && val.length() > 0 ) {
				try {
					prm.setMin( prm.getFormat().parse( val ));
				} catch ( ParseException ex ) {
				}
			} else {
				prm.setMin( null );
			}
			val = (String )cols.get( 4 );
			if ( val != null && val.length() > 0 ) {
				try {
					prm.setMax( prm.getFormat().parse( val ));
				} catch ( ParseException ex ) {
				}
			} else {
				prm.setMax( null );
			}
			prm.setConnector( con );
			prm.setChecker( chk );
			listParam.add( prm );
			GraphBufferFactory.addBuffer( con, chk, prm );
		}
		nRes = 1;
		setVisible( false );
	}

	public void onCancel() {
		if ( table.getCellEditor() != null ) {
			table.getCellEditor().stopCellEditing();
		}
		nRes = 0;
		setVisible( false );
	}

	public int getResult() {
		return nRes;
	}

	class ParamModel extends DefaultTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = new String[5];

		ParamModel() {
			cols[0] = res.getString( "graph.setup.column1.text" );
			cols[1] = res.getString( "graph.setup.column2.text" );
			cols[2] = res.getString( "graph.setup.column3.text" );
			cols[3] = res.getString( "graph.setup.column4.text" );
			cols[4] = res.getString( "graph.setup.column5.text" );
		}
		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public boolean isCellEditable( int row, int col ) {
			if ( col == 0 ) {
				return false;
			}
			return true;
		}
	}

	class ParamRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int row, int column )
		{
			Color col = (Color )value;
			setBackground( col );
			return this;
		}
	}

}
