package	jp.co.sds_corp.hibiya.monitor;

import java.awt.Component;
import java.awt.Window;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.SimpleDateFormat;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IChecker}のイベントの履歴を表示します。
*/
public class HistoryPanel extends JPanel implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	Window			owner;
	SetupParam		setup;
	ResourceBundle	res;

	HistoryModel	model;
	JTable			table;
	java.util.List<CheckerChangeEvent>	listHist;
	boolean			bFixList = false;
	Icon[]	icon = new Icon[4];
	JTextField		TXT_DATE = new JTextField();
	JTextField		TXT_TIME = new JTextField();
	JTextArea		TXT_MSG = new JTextArea( 18, 30 );
	JDialog			descDlg;

	public HistoryPanel( Window owner, SetupParam setup, java.util.List<CheckerChangeEvent> list ) {
		this( owner, setup );
		this.listHist = list;
		bFixList = true;
	}

	public HistoryPanel( Window owner, SetupParam setup ) {
		this.owner = owner;
		this.setup = setup;
		createControls();
		createIcon();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ));
		model = new HistoryModel();
		table = new JTable( model );
		JScrollPane sp = new JScrollPane( table );
		add( sp );
		table.addMouseListener( new MouseAdapter() {
        	public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					showDesc();
				}
			}
		});
		table.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent ev ) {
				if ( ev.getKeyChar() == '\n' ) {
					showDesc();
					ev.consume();
				}
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		TableColumn col;
		col = table.getColumnModel().getColumn( 0 );
		col.setPreferredWidth( 21 );
		col.setMaxWidth( 21 );
		col.setMinWidth( 21 );
		col.setCellRenderer( new StatCellRenderer() );
		col = table.getColumnModel().getColumn( 1 );
		col.setPreferredWidth( 75 );
		col = table.getColumnModel().getColumn( 2 );
		col.setPreferredWidth( 75 );
		col = table.getColumnModel().getColumn( 3 );
		col.setPreferredWidth( 300 );
		WindowUtility.removeTableEscape( table );

		descDlg = createDescDialog();
	}

	protected JDialog createDescDialog() {
		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));

		Box box1 = new Box( BoxLayout.X_AXIS );
		JLabel lbl1 = new JLabel( "Date:" );
		box1.add( lbl1 );
		TXT_DATE.setEditable( false );
		box1.add( TXT_DATE );
		box1.add( Box.createHorizontalStrut( 10 ));

		JLabel lbl2 = new JLabel( "Time:" );
		box1.add( lbl2 );
		TXT_TIME.setEditable( false );
		box1.add( TXT_TIME );
		box1.add( Box.createHorizontalStrut( 200 ) );
		pnl.add( box1 );

		JScrollPane sp = new JScrollPane( TXT_MSG );
		sp.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
		// TXT_MSG.setLineWrap( true );
		TXT_MSG.setEditable( false );
		pnl.add( sp );
		pnl.add( Box.createVerticalStrut( 5 ));

		String title = res.getString( "checker.history.title" );
		JDialog ret = new JOptionPane( pnl ).createDialog( this, title );
		ret.setResizable( true );
		ret.setSize( 477, 421 );
		return ret;
	}

	protected void createIcon() {
		int size = 18;
		icon[0] = setup.getSizedIcon( IChecker.Status.INIT, size );
		icon[1] = setup.getSizedIcon( IChecker.Status.OK, size );
		icon[2] = setup.getSizedIcon( IChecker.Status.WARN, size );
		icon[3] = setup.getSizedIcon( IChecker.Status.NG, size );
	}

	protected void setHistoryList( java.util.List<CheckerChangeEvent> list ) {
		this.listHist = list;
		((HistoryModel )table.getModel()).fireTableDataChanged();
	}

	protected void showDesc() {
		int nRow = table.getSelectedRow();
		if ( nRow < 0 ) {
			return;
		}
		CheckerChangeEvent ev = (CheckerChangeEvent )listHist.get( nRow );
		String date = (String )table.getValueAt( nRow, 1 );
		String time = (String )table.getValueAt( nRow, 2 );
		String msg = (String )table.getValueAt( nRow, 3 );
		TXT_DATE.setText( date );
		TXT_TIME.setText( time );
		Throwable ex = ev.getException();
		if ( ex != null ) {
			msg += "\n";
			for ( ;; ) {
				msg += ex.getClass().getName() + ": " + ex.getMessage() + "\n";
				StackTraceElement[] st = ex.getStackTrace();
				for ( int i = 0; i < st.length; i++ ) {
					msg += "\tat " + st[i].toString() + "\n";
				}
				ex = ex.getCause();
				if ( ex == null ) {
					break;
				}
			}
		}
		TXT_MSG.setText( msg );
		TXT_MSG.setCaretPosition( 0 );
		WindowUtility.center( this, descDlg );
		descDlg.setVisible( true );
	}

	/**
	* ICheckerの状態に変更があった場合に呼び出されます。
	* @param    ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		if ( !bFixList ) {
			IChecker chk = (IChecker )ev.getSource();
			listHist = chk.listHistory();
			if ( listHist == null ) {
				return;
			}
		}
		int num = listHist.size();
		((HistoryModel )table.getModel()).fireTableRowsInserted( num, num );
	}

	public void updateUI() {
		super.updateUI();
		if ( descDlg != null ) {
			SwingUtilities.updateComponentTreeUI( descDlg );
		}
	}

	class HistoryModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = { " ", "Date", "Time", "Message" };
		SimpleDateFormat df1 = new SimpleDateFormat( "yyyy/MM/dd" );
		SimpleDateFormat df2 = new SimpleDateFormat( "HH:mm:ss" );

		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public int getRowCount() {
			if ( listHist == null ) {
				return 0;
			}
			return listHist.size();
		}
		public Object getValueAt( int row, int col ) {
			CheckerChangeEvent ev = (CheckerChangeEvent )listHist.get( row );
			switch ( col ) {
			case 0: return ev;
			case 1: return df1.format( ev.getWhen());
			case 2: return df2.format( ev.getWhen());
			case 3: return ev.getMessage();
			}
			return null;
		}
	}

	class StatCellRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable tbl, Object obj,
			boolean isSelected, boolean hasFocus, int row, int clm )
		{
			HistoryModel model = (HistoryModel )tbl.getModel();
			CheckerChangeEvent ev
				= (CheckerChangeEvent )model.getValueAt( row, 0 );
			Icon i = null;
			switch ( ev.getNewStatus()) {
			case INIT: i = icon[0]; break;
			case OK: i = icon[1]; break;
			case WARN: i = icon[2]; break;
			case NG: i = icon[3]; break;
			default:
				break;
			}
			setIcon( i );
			return this;
		}
	}

}

