package jp.co.sds_corp.hibiya.monitor;

import java.awt.Font;
import java.awt.Rectangle;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Icon;

import jp.co.sds_corp.hibiya.action.IAction;
import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.monitor.action.IActionPanel;
import jp.co.sds_corp.hibiya.monitor.connector.IConnectorPanel;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.window.NullIcon;
import jp.co.sds_corp.util.window.SizedIcon;

/**
* Monitor全体の設定を保持します。
*/
public class SetupParam implements IConfigurable, IConfigSavable
{
	class Registry {
		String type, cls, panel;
		Registry( String type, String cls, String panel ) {
			this.type = type;
			this.cls = cls;
			this.panel = panel;
		}
		public String getType() {
			return type;
		}
		public String getOrgClass() {
			return cls;
		}
		public String getPanel() {
			return panel;
		}
	}
	HashMap<String,Registry> mapAction = new HashMap<String,Registry>();
	HashMap<String,Registry> mapActPnl = new HashMap<String,Registry>();
	HashMap<String,Registry> mapConnector = new HashMap<String,Registry>();
	HashMap<String,Registry> mapConPnl = new HashMap<String,Registry>();
	int		nVol = 100;
	int		nWidth = 700;
	int		nHeight = 430;
	int		nX = 178;
	int		nY = 215;
	int		nDivider = 150;
	int		nRowHeight = 17;
	Font	font;
	Font	defFont;
	String[] iconName = new String[4];
	String[] iconURL = {
		"/resource/init.gif",
		"/resource/ok.gif",
		"/resource/warn.gif",
		"/resource/ng.gif"
	};

	public void init( IConfiguration config ) throws Exception {
		IConfiguration c = config.getAttributes();
		nVol = c.getInteger( "Volume" );
		nX = c.getInteger( "X" );
		nY = c.getInteger( "Y" );
		nWidth = c.getInteger( "Width" );
		nHeight = c.getInteger( "Height" );
		nDivider = c.getInteger( "Divider", 150 );
		nRowHeight = c.getInteger( "RowHeight", 17 );
		iconName[0] = c.get( "OkIcon", null );
		iconName[1] = c.get( "InitIcon", null );
		iconName[2] = c.get( "ErrIcon", null );

		c = config.getChild( "Font" );
		if ( c != null ) {
			String name = c.getAttributes().get( "name" );
			int nStyle;
			String style = c.getAttributes().get( "style", "PLAIN" );
			if ( style.equals( "ITALIC" )) {
				nStyle = Font.ITALIC;
			} else if ( style.equals( "BOLD" )) {
				nStyle = Font.BOLD;
			} else if ( style.equals( "BOLD+ITALIC" )) {
				nStyle = Font.BOLD + Font.ITALIC;
			} else {
				nStyle = Font.PLAIN;
			}
			int nSize = c.getAttributes().getInteger( "size", 14 );
			font = new Font( name, nStyle, nSize );
		}

		Iterator<IConfiguration> itr = config.getChilds( "Action" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			Registry reg = new Registry(
							c.getAttributes().get( "type" ),
							c.getAttributes().get( "action" ),
							c.getAttributes().get( "panel" ) );
			mapAction.put( reg.cls, reg );
		}
		itr = config.getChilds( "Connector" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			Registry reg = new Registry(
							c.getAttributes().get( "type" ),
							c.getAttributes().get( "class" ),
							c.getAttributes().get( "panel" ) );
			mapConnector.put( reg.cls, reg );
		}
	}

	public URL getIconURL( IChecker.Status stat ) {
		int nPos = -1;
		switch ( stat ) {
		case INIT : nPos = 0; break;
		case OK : nPos = 1; break;
		case WARN : nPos = 2; break;
		case NG : nPos = 3; break;
		default:
			break;
		}
		URL img = getClass().getResource( iconURL[nPos] );
		return img;
	}

	public Icon getSizedIcon( IChecker.Status stat ) {
		return getSizedIcon( stat, 30 );
	}
	public Icon getSizedIcon( IChecker.Status stat, int nSize ) {
		Icon i = null;
		int		nPos = -1;
		switch ( stat ) {
		case INIT : nPos = 0; break;
		case OK : nPos = 1; break;
		case WARN : nPos = 2; break;
		case NG : nPos = 3; break;
		default:
			break;
		}
		if ( nPos >= 0 ) {
			if ( iconName[nPos] == null ) {
				URL img = getIconURL( stat );
				i = new SizedIcon( img, nSize );
			} else {
				i = new SizedIcon( iconName[nPos], nSize );
			}
		}
		if ( i == null ) {
			i = new NullIcon();
		}
		return i;
	}

	public void setVolume( int nVol ) {
		this.nVol = nVol;
	}
	public int getVolume() {
		return nVol;
	}

	public void saveConfig( IConfiguration config ) {
		config.setAttribute( "Volume", String.valueOf( nVol ));
		config.setAttribute( "Width", String.valueOf( nWidth ));
		config.setAttribute( "Height", String.valueOf( nHeight ));
		config.setAttribute( "X", String.valueOf( nX ));
		config.setAttribute( "Y", String.valueOf( nY ));
		config.setAttribute( "Divider", String.valueOf( nDivider ));
		config.setAttribute( "RowHeight", String.valueOf( nRowHeight ));
		IConfiguration c;
		if ( font != null ) {
			c = config.addChild( "Font" );
			c.setAttribute( "name", font.getName() );
			c.setAttribute( "size", String.valueOf( font.getSize()) );
			String s;
			if ( font.isItalic()) {
				if ( font.isBold()) {
					s = "BOLD+ITALIC";
				} else {
					s = "ITALIC";
				}
			} else {
				if ( font.isBold()) {
					s = "BOLD";
				} else {
					s = "PLAIN";
				}
			}
			c.setAttribute( "style", s );
		}

		Iterator<String> itr;
		itr = mapAction.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			Registry reg = (Registry )mapAction.get( key );
			c = config.addChild( "Action" );
			c.setAttribute( "type", reg.getType() );
			c.setAttribute( "action", reg.getOrgClass() );
			c.setAttribute( "panel", reg.getPanel() );
		}
		itr = mapConnector.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			Registry reg = (Registry )mapConnector.get( key );
			c = config.addChild( "Connector" );
			c.setAttribute( "type", reg.getType() );
			c.setAttribute( "class", reg.getOrgClass() );
			c.setAttribute( "panel", reg.getPanel() );
		}
	}

	public void setWidth( int nWidth ) {
		this.nWidth = nWidth;
	}
	public int getWidth() {
		return nWidth;
	}

	public void setHeight( int nHeight ) {
		this.nHeight = nHeight;
	}
	public int getHeight() {
		return nHeight;
	}

	public void setX( int nX ) {
		this.nX = nX;
	}
	public int getX() {
		return nX;
	}

	public void setY( int nY ) {
		this.nY = nY;
	}
	public int getY() {
		return nY;
	}

	public void setDivider( int nDivider ) {
		this.nDivider = nDivider;
	}
	public int getDivider() {
		return nDivider;
	}

	public void setRowHeight( int nRowHeight ) {
		this.nRowHeight = nRowHeight;
	}
	public int getRowHeight() {
		return nRowHeight;
	}

	public Font getDefaultFont() {
		if ( defFont == null ) {
			defFont = new Font( "Dialog", Font.PLAIN, 14 );
		}
		return defFont;
	}
	public Font getFont() {
		if ( font == null ) {
			font = getDefaultFont();
		}
		return font;
	}
	public void setFont( Font var ) {
		this.font = var;
	}

	public void setBounds( Rectangle rect ) {
		nX = (int )rect.getX();
		nY = (int )rect.getY();
		nWidth = (int )rect.getWidth();
		nHeight = (int )rect.getHeight();
	}

	/**
	* 登録されている{@link IAction}の一覧を取得します。
	*/
	public Map<String,Registry> getActionMap() {
		return mapAction;
	}
	/**
	* {@link IAction}と対応する{@link IActionPanel}のタイプ名を取得します。
	*/
	public Registry getActionType( IAction action ) {
		return (Registry )mapAction.get( action.getClass().getName() );
	}
	/**
	* {@link IAction}と対応する{@link IActionPanel}を取得します。
	*/
	public IActionPanel getActionPanel( IAction action ) throws Exception {
		String cls = action.getClass().getName();
		IActionPanel ret = (IActionPanel )mapActPnl.get( cls );
		if ( ret == null ) {
			Registry reg = (Registry )mapAction.get( cls );
			if ( reg == null ) {
				return null;
			}
			String pnlCls = reg.panel;
			ret = (IActionPanel )Class.forName( pnlCls ).newInstance();
		}
		return ret;
	}

	/**
	* 登録されている{@link IConnector}の一覧を取得します。
	*/
	public Map<String,Registry> getConnectorMap() {
		return mapConnector;
	}
	/**
	* {@link IConnector}と対応する{@link IConnectorPanel}のタイプ名を取得します。
	*/
	public Registry getConnectorType( IConnector con ) {
		return (Registry )mapConnector.get( con.getClass().getName() );
	}
	/**
	* {@link IConnector}と対応する{@link IConnectorPanel}を取得します。
	*/
	public IConnectorPanel getConnectorPanel( IConnector con )
		throws Exception
	{
		String cls = con.getClass().getName();
		IConnectorPanel ret = (IConnectorPanel )mapConPnl.get( cls );
		if ( ret == null ) {
			Registry reg = (Registry )mapConnector.get( cls );
			if ( reg == null ) {
				return null;
			}
			String pnlCls = reg.panel;
			ret = (IConnectorPanel )Class.forName( pnlCls ).newInstance();
		}
		return ret;
	}

}
