package	jp.co.sds_corp.hibiya.monitor;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IChecker}のイベントの状態を表示します。
*/
public class StatusPanel extends JPanel implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	Window			owner;
	SetupParam		setup;
	ResourceBundle	res;
	IChecker		chk;

	JTable		table;
	JTextField	TXT_KEY = new JTextField();
	JTextArea	TXT_VAL = new JTextArea( 5, 30 );
	JDialog		valDlg;

	public StatusPanel( Window owner, SetupParam setup ) {
		this.owner = owner;
		this.setup = setup;
		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));

		table = new JTable( new CheckerModel() );
		table.addMouseListener( new MouseAdapter() {
			public void mouseReleased( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					sheetClicked( ev );
				}
			}
			public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					sheetClicked( ev );
				}
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		table.getColumnModel().getColumn( 0 ).setPreferredWidth( 50 );
		table.getColumnModel().getColumn( 1 ).setPreferredWidth( 200 );
		WindowUtility.removeTableEscape( table );

		JScrollPane sp = new JScrollPane( table );
		add( sp );
		valDlg = createValueDialog();
	}

	private JDialog createValueDialog() {
		JPanel pnl = new JPanel();
		JLabel lbl1 = new JLabel( "Key:  " );
		JLabel lbl2 = new JLabel( "Value:" );

		TXT_KEY.setMaximumSize( new Dimension( 1000, 24 ));
		TXT_KEY.setEditable( false );
		TXT_VAL.setEditable( false );
		JScrollPane sp = new JScrollPane( TXT_VAL );
		TXT_VAL.setLineWrap( true );

		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));
		Box box1 = new Box( BoxLayout.X_AXIS );
		box1.add( lbl1 );
		box1.add( TXT_KEY );
		pnl.add( box1 );

		Box box2 = new Box( BoxLayout.X_AXIS );
		box2.add( lbl2 );
		box2.add( sp );
		pnl.add( box2 );

		String title = res.getString( "checker.value.title" );
		JDialog ret = new JOptionPane( pnl ).createDialog( this, title );
		ret.setResizable( true );
		ret.setSize( 450, 210 );
		return ret;
	}

	protected void sheetClicked( MouseEvent ev ) {
		int nRow = table.getSelectedRow();
		if ( nRow < 0 ) {
			return;
		}
		String key = (String )table.getValueAt( nRow, 0 );
		String value = null;
		if ( chk.getResults().get( key ) != null ) {
			value = chk.getResults().get( key ).toString();
		}
		TXT_KEY.setText( key );
		TXT_VAL.setText( value );
		TXT_VAL.setCaretPosition( 0 );
		WindowUtility.center( this, valDlg );
		valDlg.setVisible( true );
	}

	public void setChecker( IChecker var ) {
		this.chk = var;
		((CheckerModel )table.getModel()).fireTableDataChanged();
	}

	/**
	* ICheckerの状態に変更があった場合に呼び出されます。
	* @param    ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		this.chk = (IChecker )ev.getSource();
		((CheckerModel )table.getModel()).fireTableDataChanged();
	}

	class CheckerModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = { "Key", "Value" };

		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int nClm ) {
			return cols[nClm];
		}

		public int getRowCount() {
			if ( chk == null || chk.getResults() == null ) {
				return 0;
			}
			return chk.getResults().size();
		}
		public Object getValueAt( int row, int col ) {
			Map<Object,Object> map = chk.getResults();
			Iterator<Object> itr = map.keySet().iterator();
			int nPos = 0;
			while ( itr.hasNext()) {
				Object key = itr.next();
				if ( nPos == row ) {
					switch ( col ) {
					case 0: return key;
					case 1: return map.get( key );
					}
					return null;
				}
				nPos++;
			}
			return null;
		}
	}

	public void updateUI() {
		super.updateUI();
		if ( valDlg != null ) {
			SwingUtilities.updateComponentTreeUI( valDlg );
		}
	}

}
