package	jp.co.sds_corp.hibiya.monitor.checker;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.monitor.ThresholdPanel;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.threshold.IThreshold;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.window.ValuePanel;

/**
* {@link ICheckerPanel}の基底クラスです。
*/
public abstract class AbstractPanel extends JPanel implements ICheckerPanel
{
	private static final long serialVersionUID = 1L;
	IConnector	connector;
	IChecker	checker;
	IChecker	testChecker;
	ValuePanel  VAL_NAME = new ValuePanel( ValuePanel.ReturnType.STRING, "Name:" );
	ValuePanel  VAL_INT = new ValuePanel( ValuePanel.ReturnType.INT, "Interval:" );
	Box			CHK_HIST;
	JCheckBox	CB_HIST;
	JButton		BTN_TEST = new JButton( "Test" );
	ResourceBundle	res;
	String	ok_title, ok_msg;
	String	ng_title, ng_msg;
	String	warn_title, warn_msg;
	ThresholdPanel	thrPnl;

	AbstractPanel() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		ok_title = res.getString( "checker.test.ok.title" );
		ok_msg = res.getString( "checker.test.ok.message" );
		warn_title = res.getString( "checker.test.warn.title" );
		warn_msg = res.getString( "checker.test.warn.message" );
		ng_title = res.getString( "checker.test.ng.title" );
		ng_msg = res.getString( "checker.test.ng.message" );

		String text;
		text = res.getString( "checker.button.history.text" );
		CB_HIST = new JCheckBox( text );
		CHK_HIST = new Box( BoxLayout.X_AXIS );
		CHK_HIST.add( CB_HIST );
		CHK_HIST.add( Box.createHorizontalGlue() );
		text = res.getString( "checker.button.test.text" );
		BTN_TEST = new JButton( text );
		BTN_TEST.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				test( e );
			}
		});
	}

	void test( ActionEvent e ) {
		try {
			checkSetup();
			setupChecker( testChecker );
			IThreshold orgThr = getChecker().getThreshold();
			if ( orgThr != null ) {
				DOMConfiguration config = DOMConfiguration.createDocument( "threshold" );
				orgThr.saveConfig( config );
				IThreshold thr = (IThreshold )orgThr.getClass().newInstance();
				thr.init( config );
				thr.setScript( thrPnl.getScript() );
				testChecker.setThreshold( thr );
			}
		} catch ( Exception ex ) {
			String title = res.getString( "checker.test.err1.title" );
			String msg = res.getString( "checker.test.err1.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
				msg, title, JOptionPane.ERROR_MESSAGE );
			return;
		}
		try {
			testChecker.check();
		} catch ( Exception ex ) {
			String title = res.getString( "checker.test.err2.title" );
			String msg = res.getString( "checker.test.err2.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
				msg, title, JOptionPane.ERROR_MESSAGE );
			return;
		}
		IChecker.Status stat = testChecker.getCheckerStatus();
		IThreshold thr = testChecker.getThreshold();
		if ( thr != null ) {
			try {
				IChecker.Status thrstat = thr.getCheckerStatus( testChecker );
				if ( thrstat.compareTo( stat ) > 0 ) {
					stat = thrstat;
				}
			} catch ( Exception ex ) {
				String title = res.getString( "checker.test.err3.title" );
				String msg = res.getString( "checker.test.err3.message" );
				if ( ex.getMessage() == null ) {
					msg += "\n" + ex.toString();
				} else {
					msg += "\n" + ex.getMessage();
				}
				JOptionPane.showMessageDialog( this,
					msg, title, JOptionPane.ERROR_MESSAGE );
				return;
			}
		}
		if ( stat == IChecker.Status.OK ) {
			JOptionPane.showMessageDialog( this,
				ok_msg + "\n", ok_title,
				JOptionPane.INFORMATION_MESSAGE );
		} else
		if ( stat == IChecker.Status.WARN ) {
			String msg = warn_msg;
			if ( testChecker.getException() != null ) {
				msg += "\n" + testChecker.getException();
			}
			JOptionPane.showMessageDialog( this, msg, warn_title,
				JOptionPane.WARNING_MESSAGE );
		} else {
			String msg = ng_msg;
			if ( testChecker.getException() != null ) {
				msg += "\n" + testChecker.getException();
			}
			JOptionPane.showMessageDialog( this, msg, ng_title,
				JOptionPane.ERROR_MESSAGE );
		}
	}

	/**
	* 監視対象のコネクタを設定します。
	* @param	var		設定するデータ
	*/
	public void setConnector( IConnector var ) {
		this.connector = var ;
	}

	/**
	* 監視対象のモニタ(monitor)を設定します。
	* @param	var		設定するデータ
	*/
	public void setChecker( IChecker var ) {
		this.checker = var ;
		VAL_NAME.setValue( var.getName() );
		CB_HIST.setSelected( var.isSaveHistory() );
		VAL_INT.setValue( String.valueOf( var.getInterval()) );
	}
	/**
	* 監視対象のモニタ(monitor)を取得します。
	* @return	monitorの内容
	*/
	public IChecker getChecker() {
		return checker;
	}

	/**
	* ThresholdPanelを設定します。
	* @param	var		設定するデータ
	*/
	public void setThresholdPanel( ThresholdPanel var ) {
		this.thrPnl = var ;
	}

	public void checkSetup() throws Exception {
		String name = (String )VAL_NAME.getValue();
		if ( name == null || name.length() == 0 ) {
			VAL_NAME.requestFocus();
			String msg = res.getString( "checker.setup.err2.message" );
			throw( new Exception( msg ));
		}
		if ( VAL_INT.getValue() == null ) {
			VAL_INT.requestFocus();
			String msg = res.getString( "checker.setup.err1.message" );
			throw( new Exception( msg ));
		}
		int interval = ((Integer )VAL_INT.getValue()).intValue();
		if ( interval <= 0 ) {
			VAL_INT.requestFocus();
			String msg = res.getString( "checker.setup.err1.message" );
			throw( new Exception( msg ));
		}
		Object chk = connector.getCheckerMap().get( name );
		if ( chk != null && chk != checker ) {
			String msg = res.getString( "checker.setup.err3.message" );
			throw( new Exception( msg ));
		}
	}

	public void setupChecker( IChecker var ) throws Exception {
		var.setName( (String )VAL_NAME.getValue() );
		var.setSaveHistory( CB_HIST.isSelected() );
		int interval = ((Integer )VAL_INT.getValue()).intValue();
		var.setInterval( interval );
	}

	public void setEnabled( boolean bEn ) {
		super.setEnabled( bEn );
		VAL_NAME.setEnabled( bEn );
		VAL_INT.setEnabled( bEn );
		CB_HIST.setEnabled( bEn );
		BTN_TEST.setEnabled( bEn );
	}

}
